@@ -10,6 +10,7 @@ from functools import partial
 from typing import Callable, Iterable, Optional, Union
 from urllib.parse import urljoin
 from copy import deepcopy
+import dataclasses
 
 from celery.canvas import Signature, _chain
 from celery.result import AsyncResult
@@ -17,7 +18,7 @@ from celery.states import REVOKED
 from contextlib import contextmanager
 from enum import Enum
 from logging.handlers import WatchedFileHandler
-from types import MethodType
+from types import MethodType, MappingProxyType
 from abc import abstractmethod
 from celery.app.task import Task
 from celery.local import PromiseProxy
@@ -196,6 +197,33 @@ def _set_taskid_in_db_key(result: AsyncResult, db, db_key):
     logger.debug(f'Key {db_key} set to {result.id}')
 
 
+class DictWillNotAllowWrites(dict):
+    def __init__(self, *args, _instrumentation_context=None, **kwargs):
+        self.context = _instrumentation_context
+        super().__init__(*args, **kwargs)
+
+    def warn(self):
+        if self.context:
+            try:
+                self.context.send_task_instrumentation_event(abog_written_to=True)
+            except Exception:
+                logger.exception('Could not send instrumentation event')
+        try:
+            raise DeprecationWarning('DEPRECATION NOTICE: self.bog does not allow assignment. '
+                                     'Please use self.abog.copy() to create a copy of the abog that you can write to. '
+                                     'This will become an aborting failure starting January 1, 2023.')
+        except DeprecationWarning as e:
+            logger.exception(e)
+
+    def __setitem__(self, *args, **kwargs):
+        self.warn()
+        super().__setitem__(*args, **kwargs)
+
+    def __delitem__(self, *args, **kwargs):
+        self.warn()
+        super().__delitem__(*args, **kwargs)
+
+
 class FireXTask(Task):
     """
     Task object that facilitates passing of arguments and return values from one task to another, to be used in chains
@@ -211,7 +239,6 @@ class FireXTask(Task):
 
         self.undecorated = undecorate(self)
         self.sig = inspect.signature(self.run)
-        self.use_cache = get_attr_unwrapped(self, 'use_cache', False)
         self._task_return_keys = self.get_task_return_keys()
         self._decorated_return_keys = getattr(self.undecorated, "_decorated_return_keys", tuple())
         if self._decorated_return_keys and self._task_return_keys:
@@ -382,6 +409,10 @@ class FireXTask(Task):
         # Task name of first task in chain. (I.E. 'task1' in module1.task1|module2.task2)
         return task_name.split('|')[0].split('.')[-1]
 
+    @classmethod
+    def get_short_name_without_orig(cls, task_name):
+        return cls.strip_orig_from_name(cls.get_short_name(task_name))
+
     @property
     def name_without_orig(self):
         return self.strip_orig_from_name(self.name)
@@ -392,7 +423,7 @@ class FireXTask(Task):
 
     @property
     def short_name_without_orig(self):
-        return self.strip_orig_from_name(self.short_name)
+        return self.get_short_name_without_orig(self.name)
 
     @property
     def called_as_orig(self):
@@ -559,7 +590,7 @@ class FireXTask(Task):
             self.context.bog.update(result)
 
         # run any post converters attached to this task
-        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=False, **self.bag)
+        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=False, **self.bag.copy())
         self.context.bog.update(converted)
 
         if isinstance(result, dict):
@@ -569,7 +600,7 @@ class FireXTask(Task):
         # give sub-classes a chance to do something with the results
         self.post_task_run(result, extra_events=extra_events)
 
-        return self.bag
+        return self.bag.copy()
 
     def convert_results_if_returns_defined_by_task_definition(self, result):
         # If @returns decorator was used, we don't need to convert since that's taken care by the decorator
@@ -607,6 +638,25 @@ class FireXTask(Task):
         return_keys = result.get('__task_return_keys', ()) + ('__task_return_keys',)
         return {k: v for k, v in result.items() if k in return_keys}
 
+    @property
+    def default_use_cache(self):
+        return getattr(self, 'use_cache', None)
+
+    def is_cache_enabled(self):
+        use_cache_value = self.default_use_cache
+        if not self.request.called_directly:
+            try:
+                request_use_cache = self.request.properties['use_cache']
+            except KeyError:
+                pass
+            else:
+                if request_use_cache is not None:
+                    if request_use_cache != self.default_use_cache:
+                        logger.debug(f'use_cache default value of {self.default_use_cache!r} for task {self.name!r} '
+                                     f'was overridden by enqueue to {request_use_cache!r}')
+                    use_cache_value = request_use_cache
+        return bool(use_cache_value)
+
     def cache_call(self):
         cache_key = self._get_cache_key()
         try:
@@ -628,7 +678,7 @@ class FireXTask(Task):
         return self._process_result(converted_result)
 
     def final_call(self, *args, **kwargs):
-        if self.use_cache:
+        if self.is_cache_enabled():
             return self.cache_call()
         else:
             return self.real_call()
@@ -641,7 +691,7 @@ class FireXTask(Task):
                                         has_returns_from_previous_task=kwargs.get('chain_depth', 0) > 0)
 
         # run any "pre" converters attached to this task
-        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=True, **self.bag)
+        converted = ConverterRegister.task_convert(task_name=self.name, pre_task=True, **self.bag.copy())
         self.context.bog.update(converted)
 
         # give sub-classes a chance to do something with the args
@@ -660,8 +710,8 @@ class FireXTask(Task):
         super(FireXTask, self).retry(*args, **kwargs)
 
     @property
-    def bag(self) -> dict:
-        return self.context.bog.get_bag()
+    def bag(self) -> MappingProxyType:
+        return MappingProxyType(self.context.bog.get_bag())
 
     @property
     def required_args(self) -> list:
@@ -729,12 +779,12 @@ class FireXTask(Task):
         return {**bound_args, **default_bound_args}
 
     @property
-    def all_args(self) -> dict:
-        return {**self.bound_args, **self.default_bound_args}
+    def all_args(self) -> MappingProxyType:
+        return MappingProxyType({**self.bound_args, **self.default_bound_args})
 
     @property
-    def abog(self) -> dict:
-        return {**self.bag, **self.default_bound_args}
+    def abog(self) -> DictWillNotAllowWrites:
+        return DictWillNotAllowWrites(_instrumentation_context=self, **self.bag, **self.default_bound_args)
 
     #######################
     # Enqueuing child tasks
@@ -1378,6 +1428,12 @@ def convert_to_serializable(obj, max_recursive_depth=10, _depth=0):
     if hasattr(obj, 'firex_serializable'):
         return obj.firex_serializable()
 
+    if dataclasses.is_dataclass(obj):
+        try:
+            obj = dataclasses.asdict(obj)
+        except TypeError:
+            pass # e.g. enums
+
     if is_jsonable(obj):
         return obj
 
